local _M = {}
local mt = { __index = _M }
local memcached = require "resty.redis"

function _M.new(redis, port, pass, redis_time)
    local instance = {
        redis_host = redis or nil,
        redis_port = port or nil,
        redis_pass = pass or nil,
        redis_time = redis_time or nil,
        memcache = nil
    }
    return setmetatable(instance, mt)
end

function _M:connect()
    self.memcache = memcached:new()
    if not self.memcache then
        ngx.log(ngx.INFO, "failed to instantiate memc: ", err)
        return
    end
    self.memcache:set_timeouts(10000, 1000, 1000) -- 1 sec
    local ok, err = self.memcache:connect(self.redis_host, self.redis_port)
    if not ok then
        ngx.log(ngx.INFO, "failed to connect: ", err)
        return
    end
    if self.redis_pass and self.redis_pass ~= "" then
        local res, err = self.memcache:auth(self.redis_pass)
        if not res then
            ngx.log(ngx.INFO, "failed to authenticate: ", err)
            return
        end
    end
end

function _M:set(key, value, redis_time)
    self:connect()
    local ok, err = self.memcache:set(key, value)
    -- self:close()
    if not ok then
        ngx.log(ngx.DEBUG, 'set '..key..','..value..' err::'..err)
        return false
    end
    if not redis_time then
        redis_time = self.redis_time
    end
    if redis_time then
        self.memcache:expire(key, redis_time)
    end
    return true
end

function _M:settime(key, value, ctime)
    self:set(key, value, tonumber(ctime))
end

function _M:get(key)
    self:connect()
    local res, flags, err = self.memcache:get(key)
    -- self:close()
    ngx.log(ngx.DEBUG, 'get '..key)
    if err then
        ngx.log(ngx.DEBUG, 'get err::'..err)
        return nil
    end
    if not res then
        return nil
    end
    return res
end

function _M:del(key)
    self:connect()
    local ok, err = self.memcache:del(key)
    -- self:close()
    if not ok then
        ngx.log(ngx.INFO, "error del:"..key..', reason:'..err)
    end
    return ok
end

function _M:close()
    if self.memcache then
        local pool_max_idle_time = 60000 --毫秒  
        local pool_size = 100 --连接池大小 
        local ok, err = pcall(function ()
            local ok, err = self.memcache:set_keepalive(pool_max_idle_time, pool_size)  
            if not ok then  
                ngx.log(ngx.INFO, "set keepalive error : ", err)  
            end 
        end) 
    end
end

return _M
