-- -*- coding: utf-8 -*-
-- @Disc    : request frequency limit

local _M = {}


local CdnConfig = require "CdnConfig"
local request_tester = require "request_tester"
local util = require "util"
-- local cache = require "cache"
local ratelimit = require "resty.ratelimit"
function _M.init()
end
function _M.filter(zone, limits, burst, block_duration)
    local cdn = CdnConfig.new()
    cdn:load_cdn(ngx.var.host)
    local lim, err = ratelimit.new(
                        tostring(zone), 
                        limits, 
                        tonumber(burst), 
                        tonumber(block_duration))
            if not lim then
                ngx.log(ngx.ERR,
                        "failed to instantiate a resty.redis.ratelimit object: ", err)
                return ngx.exit(500)
            end
            local red
            if cdn.configs['redis_pass'] == "" then
                red = { host = cdn.configs['redis'], port = cdn.configs['redis_port'], timeout = 1 }
            else
                red = { host = cdn.configs['redis'], port = cdn.configs['redis_port'], pass = cdn.configs['redis_pass'], timeout = 1 }
            end
            
            local key = ngx.var.binary_remote_addr
            local delay, err = lim:incoming(key, red)
            if not delay then
                if err == "rejected" then
                    ngx.var.reject = "yes"
                    ngx.var.reject_reason = "ratelimit"
                    -- 拒绝
                    return ngx.exit(503)
                end
                ngx.var.reject = "yes"
                ngx.var.reject_reason = "failed to limit req"
                ngx.log(ngx.ERR, "failed to limit req: ", err)
                return ngx.exit(500)
            end

            if delay >= 0.001 then
                -- the 2nd return value holds the number of excess requests
                -- per second for the specified key.
                local excess = err

                ngx.sleep(delay)
            end
end


function _M.challenge_filter(zone, limits, burst, block_duration, ck)
    local cdn = CdnConfig.new()
    cdn:load_cdn(ngx.var.host)
    ngx.log(ngx.DEBUG, "zone,,,"..zone..','..limits..','..burst..','..block_duration..','..ck)
    local lim, err = ratelimit.new(
                        tostring(zone), 
                        limits, 
                        tonumber(burst), 
                        tonumber(block_duration))
            if not lim then
                ngx.log(ngx.ERR,
                        "failed to instantiate a resty.redis.ratelimit object: ", err)
                return ngx.exit(502)
            end
            local red
            if cdn.configs['redis_pass'] == "" then
                red = { host = cdn.configs['redis'], port = cdn.configs['redis_port'], timeout = 1 }
            else
                red = { host = cdn.configs['redis'], port = cdn.configs['redis_port'], pass = cdn.configs['redis_pass'], timeout = 1 }
            end
            
            -- local key = ngx.var.binary_remote_addr
            local delay, err = lim:incoming(ck, red)
            if not delay then
                ngx.log(ngx.DEBUG, "incoming:"..ck..','..err)
                if err == "rejected" then
                    ngx.var.reject = "yes"
                    ngx.var.reject_reason = "ratelimit"
                    -- cache.del("verify_"..ck)
                    
                    -- 不拒绝，但是删除ck
                    -- return ngx.exit(503)
                    return false
                end
                ngx.var.reject = "yes"
                ngx.var.reject_reason = "failed to limit req"
                    ngx.log(ngx.DEBUG, "incoming:"..ck..','..err)
                ngx.log(ngx.ERR, "failed to limit req: ", err)
                -- cache.del("verify_"..ck)
                -- return ngx.exit(500)
                return false
            end
            if delay >= 3 then
                ngx.log(ngx.DEBUG, "incoming:"..ck..', delay:'..delay)
                -- the 2nd return value holds the number of excess requests
                -- per second for the specified key.
                -- local excess = err
                -- return false
                -- ngx.log(ngx.DEBUG, "delay:"..delay)
                -- return ngx.exit(429)
                -- ngx.sleep(delay)
            end
            return true
end

return _M
