-- -*- coding: utf-8 -*-
-- @Date    : 2016-02-29 
-- @Author  : Alexa (AlexaZhou@163.com)
-- @Link    : 
-- @Disc    : some tools

local json = require "json"
local ffi = require("ffi")
local bit = require("bit")
ffi.cdef[[
size_t strnlen(const char *s, size_t maxlen);
]]
local _M = {}

local function chsize(char)
	if not char then
		print("not char")
		return 0
	elseif char > 240 then
		return 4
	elseif char > 225 then
		return 3
	elseif char > 192 then
		return 2
	else
		return 1
	end
end
function _M.utf8sub(str, startChar, numChars)
	local startIndex = 1
	while startChar > 1 do
		local char = string.byte(str, startIndex)
		startIndex = startIndex + chsize(char)
		startChar = startChar - 1
	end

	local currentIndex = startIndex

	while numChars > 0 and currentIndex <= #str do
		local char = string.byte(str, currentIndex)
		currentIndex = currentIndex + chsize(char)
		numChars = numChars -1
	end
	return str:sub(startIndex, currentIndex - 1)
end
function _M.block_page()
    local page = require "page"
    ngx.var.reject = "yes"
    ngx.var.reject_reason = "block"
    ngx.header['Content-Type'] = 'text/html'
    ngx.header.content_type = "text/html;charset=utf8"
	ngx.header.Cache_Control = "no-cache"
    ngx.say(page.get('blkPage'))
    ngx.exit(ngx.HTTP_OK)
    return                                
end
function _M.lan_ip(ip)

end
function _M.real_ip()
    local client_ip = ngx.var.remote_addr  -- 默认方式，获取直接连接到服务器的IP

    -- 检查X-Real-IP头（如果您的反向代理设置了这个头）
    if ngx.var.http_x_real_ip then
        client_ip = ngx.var.http_x_real_ip
    end
    
    -- 或者检查X-Forwarded-For头（如果您的反向代理设置了这个头）
    if ngx.var.http_x_forwarded_for then
        client_ip = string.gmatch(ngx.var.http_x_forwarded_for, "([^, ]+)")()  -- 获取第一个IP，因为X-Forwarded-For可能包含多个IP
    end
    return client_ip
end

function _M.isModuleAvailable(name)
  if package.loaded[name] then
    return true
  else
    for _, searcher in ipairs(package.searchers or package.loaders) do
      local loader = searcher(name)
      if type(loader) == 'function' then
        package.preload[name] = loader
        return true
      end
    end
    return false
  end
end
function _M.get_full_url()
    if ngx.var.args ~= nil then
        return ngx.var.scheme.."://"..ngx_var.host..ngx_var.uri.."?"..ngx_var.args
    else
        return ngx.var.scheme.."://"..ngx_var.host..ngx_var.uri
    end

end
function _M.get_client_ip()
    local headers=ngx.req.get_headers()
    local ip=headers["X-REAL-IP"] or headers["X_FORWARDED_FOR"] or ngx.var.remote_addr or "0.0.0.0"
    return ip
end
function _M.error(content)
    print(content)
    --ngx.header['Content-Type'] = 'text/html'
    --ngx.say(content)
    --ngx.exit(503)
end
function _M.file_exists(path)
  local file = io.open(path, "rb")
  if file then file:close() end
  return file ~= nil
end
function _M.split(s, p)
    local rt= {}
    string.gsub(s, '[^'..p..']+', function(w) table.insert(rt, w) end )
    return rt
end

function random(n, m)
    math.randomseed(os.clock()*math.random(1000000,90000000)+math.random(1000000,90000000))
    return math.random(n, m)
end

function _M.randomNumber(len)
    local rt = ""
    for i=1,len,1 do
        if i == 1 then
            rt = rt..random(1,9)
        else
            rt = rt..random(0,9)
        end
    end
    return rt
end

function _M.randomLetter(len)
    local rt = ""
    for i = 1, len, 1 do
        rt = rt..string.char(random(97,122))
    end
    return rt
end


function _M.clearHTML(html)
    html = string.gsub(html, '<script[%a%A]->[%a%A]-</script>', '')
    html = string.gsub(html, '<style[%a%A]->[%a%A]-</style>', '')
    html = string.gsub(html, '<[%a%A]->', '')
    --删除空行
    html = string.gsub(html, '\n\r', '\n')
    html = string.gsub(html, '%s+\n', '\n')
    html = string.gsub(html, '\n+', '\n')
    html = string.gsub(html, '\n%s+', '\n')
    --删除前后空格
    html = string.gsub(html, '^%s+', '')
    html = string.gsub(html, '%s+$', '')

    return html
end

function _M.json_decode( str )
    local decode = require("cjson").decode
    local ok, t = pcall(decode, str)
    if not ok then
      return nil
    end

    return t
end

function _M.tablekIn(tbl, key)
    if tbl == nil then
        return false
    end
    for k, v in pairs(tbl) do
        if k == key then
            return true
        end
    end
    return false
end
function _M.urlEncode(s)  
     s = string.gsub(s, "([^%w%.%- ])", function(c) return string.format("%%%02X", string.byte(c)) end)  
    return string.gsub(s, " ", "+")  
end  

function _M.urlDecode(s)  
    s = string.gsub(s, '%%(%x%x)', function(h) return string.char(tonumber(h, 16)) end)  
    return s  
end  

function _M.string_replace(s, pattern, replace, times)
    local ret = nil
    while times > 0 do
        times =  times - 1
        local s_start,s_stop = string.find(s, pattern , 1, true ) -- 1,true means plain searches from index 1
        if s_start ~= nil and s_stop ~= nil then 
            s = string.sub( s, 1, s_start-1 ) .. replace .. string.sub( s, s_stop+1 )
        end
    end
    return s
end

function _M.truncateStringUTF8_FFI(str, maxLength)
    if not str or not maxLength then
        return nil
    end

    local cstr = ffi.new("char[?]", #str + 1, str)
    local len = tonumber(ffi.C.strnlen(cstr, maxLength))

    -- 确保不会在UTF-8字符的中间截断
    while len > 0 and bit.band(string.byte(str, len), 0xC0) == 0x80 do
        len = len - 1
    end

    return string.sub(str, 1, len)
end

function _M.truncateString(str, maxLength)
    if not str or not maxLength then
        return nil
    end
    
    if #str <= maxLength then
        return str
    else
        return string.sub(str, 1, maxLength)
    end
end

function _M.convertToNgxPattern(str)
    -- 转换点为\. 
    str = str:gsub("%.", "\\.")
    
    -- 转换*为(.*)
    str = str:gsub("%*", "(.*)")
    
    return str
end


function _M.existed( list, value )
    for idx,item in ipairs( list ) do
        if item == value then
            return true
        end
    end
    return false
end

function _M.ngx_ctx_dump()
    local dump_str = json.encode( ngx.ctx )
    ngx.var.vn_ctx_dump = dump_str
end

function _M.ngx_ctx_load()
    
    if ngx.var.vn_ctx_dump == nil then
        return
    end

    local dump_str = ngx.var.vn_ctx_dump
    if dump_str ~= '' then
        ngx.ctx = json.decode( dump_str ) 
    end
end
function _M.get_request_args()
    local args = ngx.req.get_uri_args()
    local post_args, err = nil

    ngx.req.read_body()
    post_args, err = ngx.req.get_post_args()
    if post_args == nil then
        return args 
    end

    for k,v in pairs(post_args) do
        args[k] = v
    end

    return args
end
function _M.http_get(url)
    local http = require "resty.http"
    local httpc = http:new()
    --print(get_aip(true))
    local res, err = httpc:request_uri(url, {
        method = "GET", 
        headers = {
            ["Content-Type"] = "application/x-www-form-urlencoded", 
        },ssl_verify = false
    })
    httpc:close()
    if not res then
        --ngx.header['Content-Type'] = 'text/html'
        -- ngx.say("fail to request ", err)
        return nil --
    end
    return res["body"]
end
function _M.string_split(input, delimiter)
    input = tostring(input)
    delimiter = tostring(delimiter)
    if (delimiter=='') then return false end
    local pos,arr = 0, {}
    -- for each divider found
    for st,sp in function() return string.find(input, delimiter, pos, true) end do
        table.insert(arr, string.sub(input, pos, st - 1))
        pos = sp + 1
    end
    table.insert(arr, string.sub(input, pos))
    return arr
end
return _M
