local util = require "util"
local libinjection = require "waf.libinjection"
local xss_engine = require "waf.xss_engine"
local CdnConfig = require "CdnConfig"
local multipart = require "waf.multipart"
local resolver = require "resty.dns.resolver"

request_uri = ngx.var.request_uri
uri = ngx.unescape_uri(ngx.var.uri)
url_data=split2(request_uri,'?')
method = ngx.req.get_method()
request_header = ngx.req.get_headers(20000)
if request_header['user-agent']==nil then
  request_header['user-agent']="Windows"
end
uri_request_args = ngx.req.get_uri_args(100000)

function libinjection_args(test, requires_data)
	if type(requires_data)~='table' then return false,"" end 
	for k,v in pairs(requires_data) do
		if type(v)=='string' then
			if  test['sql'] then 
        		local issqli, fingerprint = libinjection.sqli(tostring(ReadFileHelper4(v)))
        			if issqli then 
        			    is_type='SQL注入'
        				error_rule = '语义分析分析出sql注入' .. ' >> ' .. tostring(k)..'='..tostring(v)
						ngx.var.reject_log = error_rule
        				return true,'sql'
        			end
			end 
		    if  test['xss'] then 
    			local isxss = xss_engine.xss(tostring(v))
    			if isxss == true then 
    			    is_type='XSS防御'
    				error_rule = '语义分析分析出xss跨站攻击' .. ' >> ' ..tostring(k)..'='..tostring(v)
					ngx.var.reject_log = error_rule
    				return true,'xss'
    			end
    		end 
		end 
    end 
    return false,""
end


local function post()
	if method == "GET" then return false end
	content_length=tonumber(request_header['content-length'])
	if content_length == nil then return false end
	local content_type = ngx.req.get_headers(20000)["content-type"]
	if not content_type then return false end 
	if type(content_type)~='string' then 
		return return_message(200,'Header Content-Type Error')
	end 
	if content_type and ngx.re.find(content_type, 'multipart',"oij") then return false end 
	ngx.req.read_body()
	request_args = ngx.req.get_post_args(1000000)
	if not request_args then
		if content_length >10000 then 
		    request_uri22=url_data[1]
			if request_uri22 ==nil then request_uri22='/' end
			ngx.var.reject = "yes"
			ngx.var.reject_reason = "post"
			ngx.var.reject_log = "缓冲区溢出"
			util.block_page()
			-- client_body_buffer_size的值调整为10240K 或者5024K
			-- local check_html = [[<html><meta charset="utf-8" /><title>Nginx缓冲区溢出</title><div>宝塔WAF提醒您,Nginx缓冲区溢出,传递的参数超过接受参数的大小,出现异常,<br>第一种解决方案:把当前url-->]]..'^'..request_uri22..[[加入到URL白名单中,如有疑问请联系官方运维QQ</br>第二种解决方案:面板-->nginx管理->性能调整-->client_body_buffer_size的值调整为10240K 或者5024K(PS:可能会一直请求失败建议加入白名单)</br></div></html>]]
			-- ngx.header.content_type = "text/html;charset=utf8"
			-- ngx.header.Cache_Control = "no-cache"
			-- ngx.say(check_html)
			-- ngx.exit(200)
		end 
		return true
	end
	if count_sieze(request_args)==0 then return false end 
	list_data={}
	if type(request_args)=='table' then
		for k,v in pairs(request_args)
		do
			if type(v)=='table' then
				table.insert(list_data,de_dict(k,v))
			end
            if type(v)=='string' then
				if not  string.find(v,'^data:.+/.+;base64,') then
					if (#v) >=400000 then
						ngx.var.reject = "yes"
						ngx.var.reject_reason = "post"
						ngx.var.reject_log = "参数值长度超过40w已被系统拦截"
						util.block_page()
					    return true
					end
				else
					kkkkk=ngx.re.match(v,'^data:.+;base64,','ijo')
					if  kkkkk then 
						if kkkkk[0] then 
							if ngx.re.match(kkkkk[0],'php') or ngx.re.match(kkkkk[0],'jsp') then 

								ngx.var.reject = "yes"
								ngx.var.reject_reason = "post"
								ngx.var.reject_log = "拦截Bae64上传php文件"
								util.block_page()
							end 
						end
					end
				end
			end
		end
	end
	if content_type and  ngx.re.find(content_type, '^application/json',"oij") and ngx.req.get_headers(20000)["content-length"] and tonumber(ngx.req.get_headers(20000)["content-length"]) ~= 0 then
		local ok ,request_args_json = pcall(function()
			return json.decode(ngx.req.get_body_data())
		end)
		if  ok and  type(request_args_json)=='table' then 
		    request_args_json=_process_json_args(request_args_json)
		    return args_urlencoded(request_args_json)
		end 
	end 
	if list_data then 
		if arrlen(list_data)>=1 then 
			for i2,v2 in ipairs(list_data) do 
				args_urlencoded(v2)
				request_args=_process_json_args(v2,request_args)
			end 
		else 
			request_args=_process_json_args(list_data,request_args)
		end 
	else
		request_args =_process_json_args(request_args)
	end
	if count_sieze(request_args)>=3000 then
		ngx.var.reject = "yes"
		ngx.var.reject_reason = "post"
		ngx.var.reject_log = "参数太多POST传递的参数数量超过3000"
		util.block_page()
	end
	args_urlencoded(request_args)
	return false
end

--deny cookie
function cookie_attack_check()
    local ok, sql = pcall(function()
			return CdnConfig.site_configs['firewall']['sql_inject']
	end)
	local ok, xss = pcall(function()
			return CdnConfig.site_configs['firewall']['xss_inject']
	end)
    config = {
		["sql"]: sql,
		["xss"]: xss
	}
	local fields, err = cookie:get_all()
	if not fields then
		ngx.log(ngx.ERR, err)
		return false, ""
	end
	return libinjection_args(config, fields)
end

--deny url
-- function url_attack_check()
--     if config_url_check == "on" then
--         local URL_RULES = get_rule('url.rule')
--         local REQ_URI = ngx.var.request_uri
--         for _,rule in pairs(URL_RULES) do
--             if rule ~="" and rulematch(REQ_URI,rule,"jo") then
--                 log_record('Deny_URL',REQ_URI,"-",rule)
--                 if config_waf_enable == "on" then
--                     waf_output()
--                     return true
--                 end
--             end
--         end
--     end
--     return false
-- end

--deny url args
function url_args_attack_check()
	args(uri_request_args)
    -- 防止溢出 local REQ_ARGS = ngx.req.get_uri_args()

    return false
end
--deny user agent
function user_agent_attack_check()
	local USER_AGENT_RULES = get_rule('useragent.rule')
	local USER_AGENT = ngx.var.http_user_agent
	if USER_AGENT ~= nil then
		for _,rule in pairs(USER_AGENT_RULES) do
			if rule ~="" and rulematch(USER_AGENT,rule,"jo") then
				ngx.var.reject = "yes"
				ngx.var.reject_reason = "UserAgent"
				ngx.var.reject_log = 'Deny_USER_AGENT'..ngx.var.request_uri,"-"..rule
				util.block_page()
				return true
			end
		end
	end   
    return false
end


local function args_urlencoded(request_args)
	local ok, sql = pcall(function()
		return CdnConfig.site_configs['firewall']['sql_inject']
	end)
	local ok, xss = pcall(function()
			return CdnConfig.site_configs['firewall']['xss_inject']
	end)
	config = {
		["sql"]: sql,
		["xss"]: xss
	}
	local is_status,types=libinjection_args(config, request_args)
    if is_status then 
		ngx.var.reject = "yes"
		ngx.var.reject_reason = types
		ngx.var.reject_log = "注入拦截"
		util.block_page()
	end 
end

local function _process_json_args(json_args,t)
	if type(json_args)~='table' then return {} end
	local t = t or {}
	for k,v in pairs(json_args) do
			if type(v) == 'table' then
		for _k,_v in pairs(v) do
				if type(_v) == "table" then
					t = _process_json_args(_v,t)

				else
						if type(t[k]) == "table" then
								table.insert(t[k],_v)

						elseif type(t[k]) == "string" then
								local tmp = {}
								table.insert(tmp,t[k])
								table.insert(tmp,_v)
								t[k] = tmp
						else

						t[k] = _v
						end
				end

		end
			else
				 if type(t[k]) == "table" then
						table.insert(t[k],v)
				elseif type(t[k]) == "string" then
						local tmp = {}
						table.insert(tmp,t[k])
						table.insert(tmp,v)
						t[k] = tmp
				else

				t[k] = v
				end
			end
	end
	return t
end
function args(uri_request_args)
	if count_sieze(uri_request_args)==0 then return false end 
	local list_data = nil
	rd_data={}
	if type(uri_request_args)=='table' then
		for k,v in pairs(uri_request_args)
		do
			if type(v)=='table' then
				table.insert(rd_data,de_dict2(k,v))
			end
		end
	end
	if rd_data then
		if arrlen(rd_data)>=1 then 
			for i,v in ipairs(rd_data) do 
				args_urlencoded(v)
				request_args=_process_json_args(v,uri_request_args)
			end
		else
			request_args=_process_json_args(rd_data,uri_request_args)
		end 
	else
		request_args =_process_json_args(uri_request_args)
	end
	if count_sieze(request_args)>=1000 then 
		ngx.var.reject = "yes"
		ngx.var.reject_reason = "ArgsTooMuch"
		ngx.var.reject_log = '参数超过1000拦截'
		util.block_page()
	end
	args_urlencoded(request_args)
end


--deny post
local function return_post_data2()
	if method ~= "POST" then return false end
	content_length=tonumber(request_header['content-length'])
	if not content_length then return false end
	local boundary = get_boundary()
	if boundary then
		ngx.req.read_body()
		local data = ngx.req.get_body_data()
		if not data then 
		   data=ngx.req.get_body_file()
			if data==nil then return false end
            data=read_file_body(data) 
		end
		if not data then return false end
		local tmp2 = ngx.re.gmatch(data,[[Content-Disposition.+filename=]],'ijo')
		ret={}
		while true do
		    local m, err = tmp2()
	      	if m then 
	      		table.insert(ret,m)
	      	else
	      		break
	      	end 
	    end
	    ret=gusb_string(ret)
	    if arrlen(ret)>=1 then  
		    for _,v in pairs(ret) do 
				if not ngx.re.match(v[0],'ContentbaotaDisposition: formbaotadata; name=".+"; filename=','ijo') and not ngx.re.match(v[0],'ContentbaotaDisposition: formbaotadata; name=”.+”; filename=','ijo') then 
					return_error(12)
				end
		    end
	    end
	    if arrlen(ret)==1 then 
	    	return 1 
	    else
	    	return 2 
	    end
	end
	return 3
end
local function cookie()
	if not config['cookie']['open'] or not is_site_config('cookie') then return false end
	if not request_header['cookie'] then return false end
	if type(request_header['cookie']) ~= "string" then return return_message(200,'Cookie ERROR') end
	if request_header['user-agent'] ==nil then return false end
    if type(request_header['user-agent']) ~= "string" then return false end
	request_cookie = string.lower(request_header['cookie'])
	if is_ngx_match(cookie_rules,request_cookie,'cookie') then
		ngx.var.reject = "yes"
		ngx.var.reject_reason = "cookie"
		ngx.var.reject_log = "恶意Cookie拦截"
		util.block_page()

		return true
	end
	return false
end

function post_attack_check()
	local ok, no_shell = pcall(function ()
		return CdnConfig.site_configs['firewall']['no_shell_upload']
	end)
	if not no_shell then return false end 
	content_length=tonumber(request_header['content-length'])
	if not content_length then return false end
	if content_length >108246867 then return false end
	if method =="POST" then
		return_post_data=return_post_data2()
		if not return_post_data then return false end 
		if return_post_data==3 then return false end 
		ngx.req.read_body()
		request_args2=ngx.req.get_body_data()
		if not request_args2 then 
		    request_args2=ngx.req.get_body_file()
			if request_args2==nil then return false end 
		    request_args2=read_file_body(request_args2)
		end
		if not request_args2  then return false end
		if not request_header['content-type'] then return false end
		if type(request_header['content-type']) ~= "string" then 
			if type(request_header['content-type']) ~= "string" then 
				return_error(13)
			end
		end
        local p, err = multipart.new(request_args2, ngx.var.http_content_type)
        if not p then
           return false 
        end
        if not ngx.re.match(ReadFileHelper(p['body']),is_substitution(ReadFileHelper(p['boundary2']))..'--$','ijo') then
           	return return_message(200,"from-data error")
        end
        site_count=0
        local array = {}
        while true do
            local part_body, name, mime, filename,is_filename,header_data = p:parse_part()
		    if header_data then 
               header_data_check=ngx.re.gmatch(header_data,[[Content-Disposition: form-data]],'ijo')
               ret={}
	            while true do
        		    local m, err = header_data_check()
        	      	if m then 
        	      		table.insert(ret,m)
        	      	else
        	      		break
        	      	end 
	            end
	            if arrlen(ret)>1 then 
	                return return_message(200,"from-data error2")
	            end 
            end 
            if not is_filename then
              break
            end
            site_count=site_count+1
			if is_filename then 
                filename_data=ngx.re.match(is_filename,'filename.+','ijo')
                if filename_data then
					
                     if ngx.re.match(filename_data[0],'php','ijo') or ngx.re.match(filename_data[0],'jsp','ijo') then 
						ngx.var.reject = "yes"
						ngx.var.reject_reason = "webshell"
						ngx.var.reject_log = "上传非法webshell被系统拦截"
						util.block_page()
					end 
						-- if not ngx.re.match(is_filename,'^Content-Disposition: form-data; name=".+"; filename=".+"Content-Type:','ijo') and not ngx.re.match(is_filename,'^Content-Disposition: form-data; name=".+"; filename=""Content-Type:','ijo') then 
						-- 	  is_type='恶意上传'
						-- 	if not ngx.re.match(is_filename,'^Content-Disposition: form-data; name="filename"$',"ijo") and not ngx.re.match(is_filename,'^Content-Disposition: form-data; name=".+"$',"ijo") then 
							    
						-- 	end 
						-- end
					
				end
				if(#is_filename)>1000 then 
					ngx.var.reject = "yes"
					ngx.var.reject_reason = "post"
					ngx.var.reject_log = "文件名过长"
					util.block_page()
                end 

            end
            if filename ~=nil then 
				is_type='webshell防御'
        	    if ngx.re.match(filename,'php','ijo') and ngx.re.match(filename,'jsp','ijo') then 
					ngx.var.reject = "yes"
					ngx.var.reject_reason = "webshell"
					ngx.var.reject_log = "非法上传文件名"
					util.block_page()
				end
				-- if ngx.re.match(filename,'name=','ijo') then return return_error(15) end 
				if (#filename)>=1000 then
					ngx.var.reject = "yes"
					ngx.var.reject_reason = "post"
					ngx.var.reject_log = "非法上传文件名长度超过1000被系统拦截"
					util.block_page()
        	    end 
            end
            if name ==nil then 
              if part_body then
                    if #part_body>30 then 
                        array[utf8sub(part_body,1,30)]=part_body
                    else
                        array[part_body]=part_body
                    end 
              end
            else
               	--if #name >300 then 
                --  return_error(16)
                -- end
               
               if filename ==nil then
                   if table_key(array,name) then
                        for i=1, 1000 do
                            if not table_key(array,name..'_'..i) then 
                                 if #name>30 then 
                                    array[utf8sub(name,1,30)..'_'..i]=part_body 
                                 else
                                     array[name..'_'..i]=part_body 
                                 end 
                                 
                                 break
                            end 
                        end 
                    else
                        if #name >30 then 
                            array[utf8sub(name,1,30)]=part_body
                        else 
                            array[name]=part_body
                        end 
                   end
	               if type(part_body)=='string' then
    					if (#part_body) >=400000 then
							ngx.var.reject = "yes"
							ngx.var.reject_reason = "post"
							ngx.var.reject_log = "参数值长度超过40w已被系统拦截"
							util.block_page()
    						return true
    					end
	    			end
                else
                    if type(part_body) =='string' and  part_body ~=nil then 
					    if ngx.re.find(part_body,[["php"]],'ijo') or ngx.re.find(part_body,[['php']],'ijo') or  ngx.re.find(part_body,[[<\?]],'ijo') or  ngx.re.find(part_body,[[phpinfo\(]],'ijo') or ngx.re.find(part_body,[[\$_SERVER]],'ijo') or ngx.re.find(part_body,[[<\?php]],'ijo') or ngx.re.find(part_body,[[fputs]],'ijo') or ngx.re.find(part_body,[[file_put_contents]],'ijo') or ngx.re.find(part_body,[[file_get_contents]],'ijo') or ngx.re.find(part_body,[[eval\(]],'ijo') or ngx.re.find(part_body,[[\$_POST]],'ijo')  or ngx.re.find(part_body,[[\$_GET]],'ijo') or ngx.re.find(part_body,[[base64_decode\(]],'ijo') or ngx.re.find(part_body,[[\$_REQUEST]],'ijo') or ngx.re.find(part_body,[[assert\(]],'ijo') or ngx.re.find(part_body,[[copy\(]],'ijo') or ngx.re.find(part_body,[[create_function\(]],'ijo') or ngx.re.find(part_body,[[preg_replace\(]],'ijo') or ngx.re.find(part_body,[[preg_filter\(]],'ijo') or ngx.re.find(part_body,[[system\(]],'ijo') or ngx.re.find(part_body,[[header_register_callback\(]],'ijo') or ngx.re.find(part_body,[[curl_init\(]],'ijo') or ngx.re.find(part_body,[[curl_error\(]],'ijo') or ngx.re.find(part_body,[[fopen\(]],'ijo')  or ngx.re.find(part_body,[[stream_context_create\(]],'ijo') or ngx.re.find(part_body,[[fsockopen\(]],'ijo')  then
					        local php_version=7
					        if  site_config[server_name]['php']~=nil then 
					            php_version=tonumber(site_config[server_name]['php'])
					        end
				            -- if btengine.php_detected(part_body,php_version)==1 then 
				            --     is_type='webshell防御'
				            --     lan_ip('upload','webshell防御.拦截木马上传,并被封锁IP')
				            -- end 
				        end 
                    end 
               end
            end
        end
        if site_count==0 then
        	if  config['from_data'] then 
        		return return_error2('','4') 
        	end
        end
        if count_sieze(array)>=3000 then

			ngx.var.reject = "yes"
			ngx.var.reject_reason = "post"
			ngx.var.reject_log = "参数太多POST传递的参数数量超过800"
			util.block_page()
		end
		if array['_method']  and array['method'] and array['server[REQUEST_METHOD]'] then
		    -- is_type='ThinkPHP攻击'
			-- lan_ip('php','拦截ThinkPHP 5.x RCE 攻击')
			ngx.var.reject = "yes"
			ngx.var.reject_reason = "post"
			ngx.var.reject_log = "参数太多POST传递的参数数量超过800"
			util.block_page()

		end
		if array['_method']  and array['method'] and array['server[]'] and array['get[]'] then
		    -- is_type='ThinkPHP攻击'
			-- lan_ip('php','拦截ThinkPHP 5.x RCE 攻击,并且被封锁IP')

			ngx.var.reject = "yes"
			ngx.var.reject_reason = "post"
			ngx.var.reject_log = "ThinkPHP攻击"
			util.block_page()
		end
		if array['_method'] and ngx.re.match(array['_method'],'construct','ijo') then
		    -- is_type='ThinkPHP攻击'
			-- lan_ip('php','拦截ThinkPHP 5.x RCE 攻击,并且被封锁IP')
			ngx.var.reject = "yes"
			ngx.var.reject_reason = "post"
			ngx.var.reject_log = "ThinkPHP5 RCE"
			util.block_page()
		end
      args_urlencoded(_process_json_args(array))
	   for i,v in pairs(array) do 
            if ngx.re.match(i,'\\\\$','ijo') then 
				ngx.var.reject = "yes"
				ngx.var.reject_reason = "post"
				ngx.var.reject_log = "非法上传请求已被系统拦截"
				util.block_page()
            end 
        end 
	end
end

function count_sieze(data)
    local count=0
    if type(data)~="table" then return count end 
	for k,v in pairs(data) 
	do
	    count=count+1
	end 
	return count
end 



function return_message(status,msg)
	ngx.header.content_type = "application/json;"
	ngx.header.Cache_Control = "no-cache"
	ngx.status = status
	ngx.say(json.encode(msg))
    ngx.exit(status)
end

local function get_boundary()
    local header = request_header["content-type"]
    if not header then return nil end
    if type(header) == "table" then
        return return_message(200,'content-type ERROR')
    end
	if header then
    	if ngx.re.find(header,[[multipart]],'ijo') then
    		if not ngx.re.match(header,'^multipart/form-data; boundary=') then 
    				return return_message(200,'content-type ERROR')
    		end
    	   multipart_data=ngx.re.match(header,'^multipart/form-data; boundary=.+')
    	   if not multipart_data then return return_message(200,"Boundary  Error") end
    	    if ngx.re.match(multipart_data[0],'""') then 
    	        return return_message(200,"Boundary Double Quotation Mark Error")
    	    end 
    		check_file=ngx.re.gmatch(multipart_data[0],[[=]],'ijo')
	        ret={}
	        while true do
    		    local m, err = check_file()
    	      	if m then 
    	      		table.insert(ret,m)
    	      	else
    	      		break
    	      	end 
	        end
	        if type(ret)~='table' then return false end 
	        if(arrlen(ret)>=2) then
	            return return_message(200,"multipart/form-data ERROR")
	        end
    		return header
    	else
    		return false
    	end 
    end 
end


function return_message(status,msg)
	ngx.header.content_type = "application/json;"
	ngx.header.Cache_Control = "no-cache"
	ngx.status = status
	ngx.say(json.encode(msg))
    ngx.exit(status)
end

local function  host_pachong(ip,id,ua_key)
	if not ip then return 33 end
	if not id then return 33 end
	if not ua_key then return 33 end
	local key_id=ngx.shared.btwaf:get(ip..'baidu')
	if key_id == nil then 
		local r,err= resolver:new{
                nameservers = {"8.8.8.8", {"114.114.114.114", 53} },
                retrans = 5,  
                timeout = 2000}
        if not r then return 1888 end
		local data11111=r:reverse_query(tostring(ip))
		if not data11111 then 
			return 33
		end
		if type(data11111)~='table' then return 1888 end
		if data11111['errcode'] then return 1888 end 
		if not data11111[1] then return 1888 end 
		if not  data11111[1]['ptrdname'] then return 1888 end  
		local types=string.find(string.lower(tostring(data11111[1]['ptrdname'])),string.lower(tostring(ua_key)))
		if types~=nil then 
		    write_file2(cpath..'/zhizhu'..id..'.json',ip.."\n")
			ngx.shared.btwaf:set(ip..'baidu',1,86400)
			return 2
		else
			return 35
		end
	else
		return 2
	end	
end




local function is_ngx_match(rules,sbody,rule_name)
	if rules == nil or sbody == nil then return false end
	if type(sbody) == "string" then
		sbody = {sbody}
	end
	
	if type(rules) == "string" then
		rules = {rules}
	end
	for k,body in pairs(sbody)
    do 
		if continue_key(k) then
			for i,rule in ipairs(rules)
			do
				if site_config[server_name] and rule_name then
					local n = i - 1
					for _,j in ipairs(site_config[server_name]['disable_rule'][rule_name])
					do
						if n == j then
							rule = ""
						end
					end
				end
				if body and rule ~="" then
					if type(body) == "string" and type(rule) == "string" then
						if ngx_match(ngx.unescape_uri(body),rule,"isjo") then
						    is_type =is_type_return(rule_name,rule,"static")
							error_rule = rule .. ' >> ' .. k .. '=' .. body.. ' >> ' .. body
							return true
						end
					elseif type(body) == "string" and type(rule) == "table" then
					    if ngx_match(ngx.unescape_uri(body),rule[1],"isjo") then
						    is_type =is_type_return(rule_name,rule[2],"ip")
							error_rule = rule[1] .. ' >> ' .. k .. '=' .. body.. ' >> ' .. body
							return true
						end
					end
				end
			end
		end
	end
	return false
end

local function is_ngx_match2(rules,sbody,rule_name)
	if rules == nil or sbody == nil then return false,is_type end
	if type(sbody) == "string" then
		sbody = {sbody}
	end
	
	
	local count =0
	local fraction=0
	if type(rules) == "string" then
		rules = {rules}
	end
	for k,body in pairs(sbody)
    do  
        
		if continue_key(k) then
			for i,rule in ipairs(rules)
			do
				if site_config[server_name] and rule_name then
					local n = i - 1
					for _,j in ipairs(site_config[server_name]['disable_rule'][rule_name])
					do
						if n == j then
							local rule = ""
						end
					end
				end
				if body and rule ~="" then
					if type(body) == "string" and type(rule) == "string" then
						if ngx_match(ngx.unescape_uri(body),rule,"isjo") then
						    is_type =is_type_return(rule_name,rule,"static")
							error_rule = rule .. ' >> ' .. k .. '=' .. body.. ' >> ' .. body
							ngx.shared.btwaf_data:incr(ip,100)
							fraction=fraction+100
							count=count+1
						end
					elseif type(body) == "string" and type(rule) == "table" then
					    if ngx_match(ngx.unescape_uri(body),rule[1],"isjo") then
						    is_type =is_type_return(rule_name,rule[2],"ip")
							error_rule = rule[1] .. ' >> ' .. k .. '=' .. body.. ' >> ' .. body
							ngx.shared.btwaf_data:incr(ip,rule[3])
							count=count+1
							fraction=fraction+rule[3]
						end
					end
				end
			end
		end
    end
    if count >=2 then return true,is_type end
    if fraction >=100 then return true,is_type end
	return false,is_type
end