local cache = require "cache"
local CdnConfig = require "CdnConfig"
local ssl = require "ngx.ssl"

local ocsp = require "ngx.ocsp"

local http = require "resty.http"


local util = require "util"
local ok, err = ssl.clear_certs()
    if not ok then
        ngx.log(ngx.ERR, "failed to clear existing (fallback) certificates")
        return ngx.exit(ngx.ERROR)
    end

local req_host = ssl.server_name()
if req_host == nil then
    ngx.exit(0)
end
local cdn = CdnConfig.new()
local c = cdn:load_cdn(req_host, false)
-- ngx.log(ngx.INFO, "get sslname:"..req_host)
--if CdnConfig.site_configs['domain'] == nil then

--cache.init(CdnConfig.configs['redis'], CdnConfig.configs['redis_port'], CdnConfig.configs['redis_pass'], CdnConfig.configs['redis_time'])
if cdn.site_configs == nil then
    ngx.exit(500)
end
    
    -- SSL 版本限制
    local setting = cdn.site_configs['setting']
    if setting and setting['ssl'] then
        --[[if setting['ssl']['HSTS'] then 
        -- 这里的htst、强制ssl都迁移到了on rewrite阶段，节约时间
        end]]--

        if setting['ssl']['SSLVersion'] then
            ngx.log(ngx.ERR, "get setting:"..setting['ssl']['SSLVersion'])
            local ver = tonumber(util.string_replace(ssl.get_tls1_version(), '0x030', '', 1))
            local low = tonumber(util.string_replace(setting['ssl']['SSLVersion'], '0x030', '', 1))
            if ver < low then
                ngx.log(ngx.ERR, "get setting ver<low:"..ver)
                ngx.exit(0)
                return
            end
        end
    end
    -- print(ssl.server_name())
--end
--print(ssl.server_name())
-- print(CdnConfig.site_configs['ssl_enable'])
local arr = cdn:load_ssl()
local der_cert_chain = nil
--ngx.log(ngx.ERR, "CDN IS :".. CdnConfig.site_configs['domain'])
if arr and #arr == 2 then
    -- crt = arr[1]
    -- key = arr[2]
    local cert, err = ssl.parse_pem_cert(arr[1])
    if not cert then
        ngx.log(ngx.ERR, "failed to parse PEM cert: ", err)
        return
    end
    
     
    --证书格式有pem 和der两种，针对pem 的证书，我们把它转换成der，方便后续做oscp stapling。
    --if ssl_https_data['type'] == 'pem' then
        der_cert_chain, err = ssl.cert_pem_to_der(arr[1])
        if not der_cert_chain then 
            ngx.log(ngx.ERR, "pem cert transe to  der cert chain faild.")
            return 
        end
     
        --der_priv_key, err = ssl.priv_key_pem_to_der(ssl_https_data['pkey_data'])
        --if not der_priv_key then 
        --    ngx.log(ngx.ERR, "Pem key transe to der private key faild.")
        --    return 
        --end
    --end

    
    local ok, err = ssl.set_cert(cert)
    if not ok then
        ngx.log(ngx.ERR, "failed to set cert: ", err)
        return
    end
    
    local pkey, err = ssl.parse_pem_priv_key(arr[2])
    if not pkey then
        ngx.log(ngx.ERR, "failed to parse pem key: ", err)
        return
    end

    local ok, err = ssl.set_priv_key(pkey)
    if not ok then
        ngx.log(ngx.ERR, "failed to set private key: ", err)
        return
    end


end


local shared_data = ngx.shared.ssl_session
if der_cert_chain == nil then
    ngx.exit(0)
    return
end
---OSCP更新。共享内存如果不存在或者已过期，则从云端更新oscp response,并update到share dict
local req_host_oscp_res = nil
--oscp key
local req_host_oscp = req_host..'_oscp'
req_host_oscp_res = shared_data:get(req_host_oscp)
if req_host_oscp_res == nil then
 
    local ocsp_url, err = ocsp.get_ocsp_responder_from_der_chain(der_cert_chain)
    if not ocsp_url then
        ngx.log(ngx.ERR, "failed to get OCSP responder: ", err)
        return ngx.exit(ngx.ERROR)
    end
    ngx.log(ngx.INFO, "oscp url:", ocsp_url)
    
    
    local ocsp_req, err = ocsp.create_ocsp_request(der_cert_chain)
    if not ocsp_req then
        ngx.log(ngx.ERR, "failed to create OCSP request: ", err)
        return ngx.exit(ngx.ERROR)
    end
    
    local httpc = http.new()
    httpc:set_timeout(10000)
    local res, req_err = httpc:request_uri(ocsp_url, {
        method = "POST",
        body = ocsp_req,
        headers = {
            ["Content-Type"] = "application/ocsp-request",
        }
    })
    
    -- 校验 CA 的返回结果
    if not res then
        ngx.log(ngx.ERR, "OCSP responder query failed: ", err)
        -- return ngx.exit(ngx.ERROR)
        req_host_oscp_res = false
    else
    
    local http_status = res.status
    
    if http_status ~= 200 then
        ngx.log(ngx.ERR, "OCSP responder returns bad HTTP status code ",
                http_status)
        -- return ngx.exit(ngx.ERROR)
    end
    
    req_host_oscp_res = res.body
    --ngx.log(ngx.INFO, 'req_host_oscp_res:', req_host_oscp_res)
    --设置ocsp 到 share dict
    local ok,err = shared_data:safe_set(req_host_oscp, req_host_oscp_res, 600)
    end
end
 
 
--if req_host_oscp_res and #req_host_oscp_res > 0 then
if req_host_oscp_res then
    local ok, err = ocsp.validate_ocsp_response(req_host_oscp_res, der_cert_chain)
    if not ok then
        ngx.log(ngx.ERR, "failed to validate OCSP response: ", err)
        -- return ngx.exit(ngx.ERROR)
    end
 
    -- 设置当前 SSL 连接的 OCSP stapling
    ok, err = ocsp.set_ocsp_status_resp(req_host_oscp_res)
    if not ok then
        ngx.log(ngx.ERR, "failed to set ocsp status resp: ", err)
        -- return ngx.exit(ngx.ERROR)
    end
end