local util = require "util"
local _M = {}
local cache = require "cache"
--The md5 of config string
_M.config_hash = nil

_M["configs"] = {}
_M["site_configs"] = {}

_M.prefix = "Site_"
_M.dprefix = "Domain_"
_M.hprefix = "Host_"
_M.sprefix = "SSL_"
_M.cprefix = "/www/config/"

_M.configPath = ''
_M.default_html = nil
function _M.showdefault()

    if _M.default_html == nil then
        local path = _M.home_path() .."/configs/index.html"
        local f = io.open( path, 'r' )
        if f ~= nil then
            _M.default_html = f:read("*all")
            f:close()
        end
    end
    local html = util.string_replace( _M.default_html,'{{eventID}}', ngx.var.eventsid, 1 )
        
        ngx.header.content_type = "text/html"
        ngx.header['cache-control'] = "no-cache, no-store, must-revalidate"
        ngx.header['pragma'] = "no-cache"
        ngx.header['expires'] = "0"
        ngx.header.charset = "utf-8"
        ngx.say( html )
        
        ngx.exit(200)

end

function _M.version_updater_035( configs )
    configs["summary_with_host"] = false
    configs["summary_group_persistent_enable"] = true
    configs["summary_group_temporary_enable"] = true
    configs["summary_temporary_period"] = 60
    configs["config_version"] = "0.36"
    return configs
end

_M.version_updater = {
    ['0.35'] = _M.version_updater_035,
}

-------------------Config Updater end---------------------


local dkjson = require "dkjson"
local json = require "json"


function _M.home_path()
    local current_script_path = debug.getinfo(1, "S").source:sub(2)
    local home_path = current_script_path:sub( 1, 0 - string.len("/script/CdnConfig.lua") -1 ) 
    return home_path
end

function _M.update_config()
    --save a hash of current in lua environment
    local new_config_hash = ngx.shared.status:get('vn_config_hash')
    if new_config_hash ~= nil and new_config_hash ~= _M.config_hash then
        ngx.log(ngx.STDERR,"config Hash Changed:now reload config from config.json")
        _M.load_server_from_file()
    end
end
function _M.getHostConfig(domain, host, types)
    if domain == nil then
        domain = ""
    end
    if host == nil then
        host = ""
    end
    -- ngx.header['Content-Type'] = 'text/html'
    -- ngx.say(_M.configs["data"] .. "?type=" .. types .. "&domain=" .. domain .. "&host=" .. host)
    local res = util.http_get(_M.configs["data"] .. "?type=" .. types .. "&domain=" .. domain .. "&host=" .. host)
    -- 加到 domain_list
    if types == "domain" then
        local domain_list = ngx.shared.domain_list:get('all')
        if domain_list == nil then
            domain_list = ""
        end
        local arr = util.split(domain_list, ',')
        local find = false
        for _, v in pairs(arr) do
            if v == domain then 
                find = true
                break
            end
        end
        -- ngx.log(ngx.DEBUG, "hi domain_list: "..domain_list..','..#arr)
        if find == false then 
            table.insert(arr, domain)
            ngx.shared.domain_list:set('all', table.concat(arr, ','))
        end
    elseif types == "all" then
        -- 更新host的时候顺便更新ssl，所以不用多写
        local domain_list = ngx.shared.host_list:get('all')
        if domain_list == nil then
            domain_list = ""
        end
        local arr = util.split(domain_list, ',')
        local find = false
        for _, v in pairs(arr) do
            if v == domain then 
                find = true
                break
            end
        end
        --ngx.log(ngx.DEBUG, "hi domain_list: "..domain_list..','..#arr)
        if find == false then 
            table.insert(arr, host)
            --  ngx.log(ngx.DEBUG, "hi table: "..domain..','..#arr)
            ngx.shared.host_list:set('all', table.concat(arr, ','))
        end
    end
    -- ngx.header['Content-Type'] = 'text/html'
    -- ngx.say(res)
    return res
end
function _M.reloadsite()
    local domain = ngx.var.arg_domain
    local host = ngx.var.arg_host
    if domain ~= nil then _M.reloadDomain(domain) end
    if host ~= nil then _M.reloadHost(host) end
end
function _M.reloadDomain(domain)
    _M.load_server_from_file()
    -- cache.init(_M.configs['redis'], _M.configs['redis_port'], _M.configs['redis_pass'], _M.configs['redis_time'])
    delCache("domain", domain)
    --[[
    data = ngx.shared.domain:set( _M.dprefix .. domain, 1, _M.configs['cache_time'] )
    ngx.shared.cdn_setting:delete( _M.prefix .. domain )
    cache.del(_M.prefix .. domain)
    _M.load_cdn(domain)
    ]]--
end
function _M.reloadHost(host)
    local host = ngx.var.arg_host
    _M.load_server_from_file()
    -- local cache_instance = require "cache" 
    -- cache.init()
    delCache("host", host)
    --[[data = ngx.shared.domain:set( _M.dprefix .. domain, 1, _M.configs['cache_time'] )
    ngx.shared.cdn_setting:delete( _M.prefix .. domain )
    cache.del(_M.prefix .. domain)
    _M.load_cdn(domain)
    ]]--
end

function _M.initcache()
    return cache.new(_M.configs['redis'], _M.configs['redis_port'], _M.configs['redis_pass'], _M.configs['redis_time'])
end
function getCache(types, name)
    local data
    if types == "domain" then
        name = _M.dprefix .. name
        data = ngx.shared.domain:get(name)
    end
    if types == "host" then
        name = _M.hprefix .. name
        data = ngx.shared.host:get(name)
    end
    if types == "ssl" then
        name = _M.sprefix .. name
        data = ngx.shared.ssl:get(name)
    end
    if data ~= nil then 
        return data
    end
    -- data = cache.get(name)
    if data ~= nil then
        --ngx.log(ngx.INFO, "data:" .. name..',data:'..data)
        if types == "domain" then
            ngx.shared.domain:set(name, data, _M.configs['cache_time'])
        end
        if types == "host" then
            ngx.shared.host:set(name, data, _M.configs['cache_time'])
        end
        if types == "ssl" then
            ngx.shared.ssl:set(name, data, _M.configs['cache_time'])
        end
        return data
    end
    return nil
end
function setCache(types, name, value)
    if types == "domain" then
        name = _M.dprefix .. name
        ngx.shared.domain:set(name, value, _M.configs['cache_time'])
        
    end
    if types == "host" then
        name = _M.hprefix .. name
        ngx.shared.host:set(name, value, _M.configs['cache_time'])
    end
    if types == "ssl" then
        name = _M.sprefix .. name
        ngx.shared.ssl:set(name, value, _M.configs['cache_time'])
    end
    local cache_instance = _M.initcache()
    cache_instance:set(name, value, _M.configs['cache_time'])
    cache_instance:close()
end
function delCache(types, name)
    if types == "domain" then
        name = _M.dprefix .. name
        ngx.shared.domain:delete(name)
    end
    if types == "host" then
        name = _M.hprefix .. name
        ngx.shared.host:delete(name)
    end
    if types == "ssl" then
        name = _M.sprefix .. name
        ngx.shared.ssl:delete(name)
    end
    local cache_instance = _M.initcache()
    cache_instance:del(name)
    cache_instance:close()
end

function _M.tryload(hostname, force)
    return tryload(hostname, force)
end
function tryload(hostname, force)
    local host = hostname
    if force == nil or not force then
    -- local name = _M.hprefix .. host
        local data = getCache("host", host)
        if data ~= nil then
            if data == 0 or data == '0' then
                -- ngx.say("no domain")
                return nil
            end
            _M.configPath = host
            data = json.decode(data)
            return data
        end
        
        local arr = util.split(host, '.')
        local domain = "#."
        if #arr >= 2 then
            for i=2,#arr do
                domain = domain .. arr[i]
            end
        end
        -- 二次判断是否存在泛解析
        data = getCache("host", domain)
        if data ~= nil then
            _M.configPath = domain
            data = json.decode(data)
            return data
        end
    end
    
    local data = _M.getHostConfig(host, host, 'all')
    --ngx.log(ngx.INFO, "res:" .. data)
    if data == "no" then 
        return nil
    end
    data = util.string_split(data, "$#;#$")   -- domain$#:#$domaindata $#;#$ host
    if #data == 2 then
        local domain = util.string_split(data[1], "$#:#$")
        ngx.log(ngx.INFO, "domain"..data[1]..'#'..#domain)
        if #domain == 2 then
            if getCache("domain", domain[1]) == nil then
                setCache("domain", domain[1], domain[2])
            end
        end
        host = util.string_split(data[2], "$#:#$")
        if #host == 2 then
            _M.configPath = host[1]
            setCache("host", host[1], host[2])
            return json.decode(host[2])
        end
    end
            
    --[[
    local arr = util.split(domain, '.')
    
    domain = arr[#arr-1] ..'.'.. arr[#arr]
    data = ngx.shared.domain:get( _M.dprefix .. domain )
    if data ~= nil then
        return domain
    end
    domain = arr[#arr-2] ..'.'.. arr[#arr-1] ..'.'.. arr[#arr]
    data = ngx.shared.domain:get( _M.dprefix .. domain )
    if data ~= nil then
        if data == 0 or data == "0" then
            return nil
        end
        return domain
    end
    
    domain = arr[#arr-1] ..'.'.. arr[#arr]
    if util.file_exists(_M.cprefix .. string.sub(domain, 1, 1) .. '/' .. domain .. '.json') then
        ngx.shared.domain:set(_M.dprefix .. domain, 1, _M.configs['cache_time'] )
        return domain
    else
        domain = arr[#arr-2] ..'.'.. arr[#arr-1] ..'.'.. arr[#arr]
        -- ngx.say(util.file_exists(_M.cprefix .. string.sub(domain, 1, 1) .. '/' .. domain .. '.json'))
        if util.file_exists(_M.cprefix .. string.sub(domain, 1, 1) .. '/' .. domain .. '.json') then
            ngx.shared.domain:set(_M.dprefix .. domain, 1, _M.configs['cache_time'] )
            return domain
        end
    end
    ]]
    -- 域名不存在，记录缓存为0，下次查找直接返回
    setCache("host", hostname, 0)
    --ngx.shared.domain:set(_M.hprefix .. host, 0, _M.configs['cache_time'] )
    return nil
end
function _M.load_ssl()
    -- local host = ngx.var.http_host
    if _M.site_configs['ssl_enable'] then
        local data = getCache('ssl', _M.site_configs['ssl'])
        if data then
            return util.split(data, "$#:#$")
        else
            data = _M.getHostConfig(_M.site_configs['domain'], _M.site_configs['ssl'], 'ssl')
            if data and data ~= "no" then
                setCache("ssl", _M.site_configs['ssl'], data)
                return util.split(data, "$#:#$")
            else
                --local url = 'http://' .. host .. ngx.var.request_uri
                --ngx.redirect(url, ngx.HTTP_MOVED_PERMANENTLY)
            end
        end
    else
        --local url = 'http://' .. host .. ngx.var.request_uri
        --ngx.redirect(url, ngx.HTTP_MOVED_PERMANENTLY)
    end
end
function _M.load_cdn(hostname, force)
    local data
    if _M.configs == nil then
        return
    end
    local host = tryload(hostname)
    -- ngx.log(ngx.INFO, "domain_id"..host['domain_id'])
    if host == nil or not host or host['domain'] == nil then
        if ngx.get_phase() == "ssl_cert" then
            
        else
            if force ~= nil and force then

            else
                _M.showdefault();
            end
            
            --[[
                print("no domain")
            ngx.header['Content-Type'] = 'text/html'
            ngx.status = ngx.HTTP_FORBIDDEN
            ngx.say("No domain, please login console.")
            -- to cause quit the whole request rather than the current phase handler
            ngx.exit(ngx.HTTP_OK)
            ]]--
        end
    end
    --[[
    -- 从内存缓存读取
    data = ngx.shared.cdn_setting:get( _M.prefix .. domain )
    if data ~= nil then
        if #data == 2 then
            ngx.header['Content-Type'] = 'text/html'
            ngx.say(domain .. " not allowed.")
            ngx.exit(200)
        end
        _M.site_configs = data
        return
    end
    
    local cache_instance = require "cache" 
    data = cache.get(_M.prefix .. domain)
    if data then
        ngx.shared.cdn_setting:set( _M.prefix .. domain, data, _M.configs['cache_time'] )
        _M.site_configs = data
        return 
    end
    ]]--
    if force == nil or not force then
        data = getCache("domain", host["domain"])
        if data ~= nil then
            data = require("json").decode(data)
            for k,v in pairs(host) do 
                --if k ~= 'domain' then
                data[k] = v 
                --end
            end -- domain host数据合并
            _M.site_configs = data
            return
        end
    end
    -- ngx.say("no cache")
    local res = _M.getHostConfig(host['domain'], "", 'domain')-- util.http_get( _M.configs["data"] .. "?type=domain&domain=" .. host['domain'])
    if res ~= nil then
        local arr = util.string_split(res, "$#:#$")
        if #arr == 2 then
            data = json.decode(arr[2])
            setCache("domain", host["domain"], arr[2])
            for k,v in pairs(host) do 
                --if k ~= 'domain' then
                data[k] = v 
                -- end
            end -- domain host数据合并，从而覆盖主站点设置
            _M.site_configs = data
        end
        return
    else
        if force ~= nil or not force then

            --pass
        else
            -- ngx.header['Content-Type'] = 'text/html'
            -- ngx.say("no domain or data is not sync.")
            -- print("no domain or proxy data")
            ngx.header['Content-Type'] = 'text/html'
            ngx.status = ngx.HTTP_FORBIDDEN
            ngx.say("no domain or data is not sync.")
            -- to cause quit the whole request rather than the current phase handler
            ngx.exit(ngx.HTTP_OK)
            return nil
        end
    end
    -- 从NFS读取
    --[[
    local file = io.open( _M.cprefix .. string.sub(domain, 1, 1) .. '/' .. domain .. '.json', "r")
    if file == nil then
        ngx.shared.cdn_setting:set( _M.prefix .. domain, "no", _M.configs['cache_time'] )
        cache.set( _M.prefix .. domain, "no" )
        
        ngx.header['Content-Type'] = 'text/html'
        ngx.say(domain .. " not allowed.")
        ngx.exit(200)
        return 
    end
    
    --file = io.open( "/tmp/config.json", "w");
    data = json.decode( file:read("*all") )
    file:close()
    ]]--
    if data ~= nil then
        ngx.shared.cdn_setting:set( _M.prefix .. domain, data, _M.configs['cache_time'] )
        local cache_instance = _M.initcache()
        cache_instance:set( _M.prefix .. domain, data )
        cache_instance:close()
        _M.site_configs = data
    end
end

function _M.load_server_from_file()
    local config_dump_path = _M.home_path() .. "/configs/config.json"
    local file = io.open( config_dump_path, "r")
    
    if file == nil then
        return json.encode({["ret"]="error",["msg"]="config file not found"})
    end

    --file = io.open( "/tmp/config.json", "w");
    local data = file:read("*all");
    file:close();

    --save config hash in module
    local config_hash = ngx.md5(data)
    _M.config_hash = config_hash
    ngx.shared.status:set('vn_config_hash', config_hash )

    --ngx.log(ngx.STDERR, data)
    local tmp = dkjson.decode( data )
    if tmp ~= nil then
        --update config version if need
        local loop = true
        while loop do
            local handle = _M.version_updater[ tmp['config_version'] ] 
            if handle ~= nil then
                tmp = handle( tmp )
            else
                loop = false
            end 
        end

        if tmp['config_version'] ~= _M["configs"]["config_version"] then
            ngx.log(ngx.STDERR,"load config from config.json error,will use default config")
            ngx.log(ngx.STDERR,"Except Version:")
            ngx.log(ngx.STDERR, _M["configs"]["config_version"] )
            ngx.log(ngx.STDERR,"Config.json Version:")
            ngx.log(ngx.STDERR,tmp["config_version"])
        else
            _M["configs"] =  tmp
        end

        --return json.encode({["ret"]="success",['config']=_M["configs"]})
    else 
        ngx.log(ngx.STDERR,"config.json decode error")
        --return json.encode({["ret"]="error",["msg"]="config file decode error, will use default"})
    end
        
end 
--[[
--return a json contain current config items
function _M.report()
    _M.set_config_metadata( _M["configs"] )
    return dkjson.encode( _M["configs"] )
end

function _M.set()
    local ret = false
    local err = nil
    local args = nil
    local dump_ret = nil

    ngx.req.read_body()
    args, err = ngx.req.get_post_args()
    if not args then
        ngx.say("failed to get post args: ", err)
        return 
    end

    local new_config_json_escaped_base64 = args['config']
    local new_config_json_escaped = ngx.decode_base64( new_config_json_escaped_base64 )
    --ngx.log(ngx.STDERR,new_config_json_escaped)
   
    local new_config_json = ngx.unescape_uri( new_config_json_escaped )
    --ngx.log(ngx.STDERR,new_config_json)

    local new_config = json.decode( new_config_json )

    if _M.configs['readonly'] == true then
        ret = false
        err = "all configs was set to readonly"
    elseif _M.verify( new_config ) == true then
        ret, err = _M.dump_to_file( new_config ) 
    end

    if ret == true then
        return json.encode({["ret"]="success",["err"]=err})
    else
        return json.encode({["ret"]="failed",["err"]=err})
    end
end

function _M.set_config_metadata( config_table )

    --make sure empty table trans to right type
    local meta_table = {}
    meta_table['__jsontype'] = 'object'
    
    if config_table['matcher'] ~= nil then
        setmetatable( config_table['matcher'], meta_table )
        for key, t in pairs( config_table["matcher"] ) do
            setmetatable( t, meta_table )
        end
    end
    
    if config_table['backend_upstream'] ~= nil then
        setmetatable( config_table['backend_upstream'], meta_table )
        for key, t in pairs( config_table["backend_upstream"] ) do
            setmetatable( t['node'], meta_table )
        end
    end
    
    if config_table['response'] ~= nil then
        setmetatable( config_table['response'], meta_table )
    end
    --set table meta_data end

end

function _M.dump_to_file( config_table )

    _M.set_config_metadata( config_table )

    local config_data = dkjson.encode( config_table , {indent=true} ) --must use dkjson at here because it can handle the metadata
    local config_dump_path = _M.home_path() .. "/configs/config.json"
    
    --ngx.log(ngx.STDERR,config_dump_path)
    local file, err = io.open( config_dump_path, "w")
    if file ~= nil then
        file:write(config_data)
        file:close()
        --update config hash in shared dict
        ngx.shared.status:set('vn_config_hash', ngx.md5(config_data) )
        return true
    else
        return false, "open file failed"
    end

end
]]--
--auto load config from json file
-- 服务端config
-- _M.load_server_from_file()

return _M
