local _M = {}

local CdnConfig = require "CdnConfig"
local request_tester = require "request_tester"
local encrypt_seed = require "encrypt_seed"
local frequency_limit = require "frequency_limit"
local util = require "util"
local ck = require("cookie")
-- local cache = require("cache")

_M.verify_javascript_html = nil
_M.verify_captcha_html = nil

function _M.sign( mark )
    local ua = ngx.var.http_user_agent
    local forwarded  = ngx.var.http_x_forwarded_for
    
    if ua == nil then
        ua = ''
    end
    
    if forwarded == nil then
        forwarded = ''
    end

    local sign = ngx.md5( 'YEAPCLOUD' .. ngx.var.remote_addr .. forwarded .. ua .. mark .. encrypt_seed.get_seed() )
    return sign 
end

function _M.getCookie()
    local cookie, err = ck:new()
    if not cookie then
        return false
    end

    -- get single cookie
    local cookie_prefix = CdnConfig.configs['cookie_prefix']
    local field, err = cookie:get(cookie_prefix)
    if not field then
        return false
    end
    return field
end

function _M.verify(vtype, duration)
    local cookie = _M.getCookie()
    if not cookie then
        return false
    end
    -- local res = cache:get("verify_"..cookie)
    local cache = CdnConfig.initcache()
    local res = cache:get("verify_"..cookie)
    if res ~= nil then
        cache:close()
        return true
    end
    local res = util.http_get("https://captcha-backend-api.qisweb.com/query_backend_api.php?token="..cookie)
    if res == "ok" then 
        -- cache:init()
        cache:settime("verify_"..cookie, "1", duration)
        cache:close()
        return true
    end
    return false
end


function _M.verify_captcha(duration)
    -- 没有通过验证前限制ratelimit
    -- 
    if ngx.var.http_cookie ~= nil then
        if(_M.verify(duration)) then
            return
        end
    end
    local sign = _M.sign('cookie')
        if _M.verify_captcha_html == nil then
            local path = CdnConfig.home_path() .."/configs/verify_captcha.html"
            local f = io.open( path, 'r' )
            if f ~= nil then
                _M.verify_captcha_html = f:read("*all")
                f:close()
            end
        end
        
        local cookie_prefix = CdnConfig.configs['cookie_prefix']
    
        local redirect_to = nil
        local html = _M.verify_captcha_html
    
        html = string.gsub( html,'INFOCOOKIE',sign )
        html = string.gsub( html,'COOKIEPREFIX',cookie_prefix )
    
        if ngx.var.args ~= nil then
            redirect_to =  ngx.var.scheme.."://"..ngx.var.http_host..ngx.var.uri.."?"..ngx.var.args , ngx.HTTP_MOVED_TEMPORARILY
        else
            redirect_to =  ngx.var.scheme.."://"..ngx.var.http_host..ngx.var.uri , ngx.HTTP_MOVED_TEMPORARILY
        end
    
        html = util.string_replace( html,'INFOURI',redirect_to, 1 )
        
        ngx.var.verified = "1"
        ngx.header.content_type = "text/html"
        ngx.header['cache-control'] = "no-cache, no-store, must-revalidate"
        ngx.header['pragma'] = "no-cache"
        ngx.header['expires'] = "0"
        ngx.header.charset = "utf-8"
        ngx.say( html )
        
        ngx.exit(200)
end
--[[
function _M.verify_cookie()
    
    -- local sign = _M.sign('challenge')
    local cookie_prefix = CdnConfig.configs['cookie_prefix']
    ngx.header["Set-Cookie"] =  cookie_prefix .. "_sign_cookie=" .. sign .. '; path=/' 
    
    if ngx.var.args ~= nil then
        ngx.redirect( ngx.var.scheme.."://"..ngx.var.http_host..ngx.var.uri.."?"..ngx.var.query_string , ngx.HTTP_MOVED_TEMPORARILY)
    else
        ngx.redirect( ngx.var.scheme.."://"..ngx.var.http_host..ngx.var.uri , ngx.HTTP_MOVED_TEMPORARILY)
    end
end
]]--


function _M.sign5s(duration)
    local ua = ngx.var.http_user_agent
    local forwarded  = ngx.var.http_x_forwarded_for
    if ua == nil then
        ua = ''
    end    
    if forwarded == nil then
        forwarded = ''
    end
    local sign = ngx.time() ..'_'..ngx.md5( 'YEAPCLOUD' .. ngx.var.remote_addr .. forwarded .. ua .. "5s" .. encrypt_seed.get_seed() )
    local cache = CdnConfig.initcache()
    cache:settime("verify_"..sign, "1", duration)
    cache:close()
    -- ngx.log(ngx.DEBUG, "sign5s:".. "verify_"..sign)
    -- ngx.log(ngx.DEBUG, "get cache:".. "verify_"..cache:get("verify_"..sign))
    
    return sign 
end

function _M.verify_javascript(duration)
    local cookie = _M.getCookie()
    if cookie ~= nil and cookie ~= false then
        cookie = tostring(cookie)
        local cache = CdnConfig.initcache()
        local res = cache:get("verify_"..cookie)
        if res ~= nil and tostring(res) == "1" then
            ngx.log(ngx.DEBUG, "[cache]cache verify_"..cookie)
            -- ngx.log(ngx.DEBUG, "res:"..res)
            local c_arr = util.string_split(cookie, '_')
            -- ngx.log(ngx.DEBUG, "ct:"..CdnConfig.site_configs['setting']['firewall']['ct'])
            if #c_arr == 2 then
                local ts = tonumber(c_arr[1])
                if ts + 5 > ngx.time() then
                    cache:del("verify_"..cookie)
                    ngx.log(ngx.DEBUG, "cookie del:"..cookie..', #ts'..ts..' +5 >'..ngx.time())
                else
                    if ts + 15 < ngx.time() then
                        -- 已经超过了，不需要续期，只做ratelimit
                        -- frequency_limit
                        local zone = CdnConfig.site_configs['domain_id'] -- "verify_" ..  .. "_" .. cookie 
                        local res = frequency_limit.challenge_filter(zone, "150r/m", 200, 15, cookie)
                        -- 超过ratelimit直接要求重新验证，防止利用ck攻击
                        if res == false then 
                            ngx.log(ngx.DEBUG, "[cache]cache del "..cookie)
                            cache:del("verify_"..cookie)
                        else
                            return true
                        end
                    else 
                        -- 还在30秒内，续期一下
                        cache:settime("verify_"..cookie, "1", duration)
                        cache:close()
                        ngx.log(ngx.DEBUG, "cookie 续期:"..cookie..', #'.. duration)
                        return true
                    end
                end
                -- 错误ck
            else
                ngx.log(ngx.DEBUG, "#c_arr:"..#c_arr)
            end
        else
            ngx.log(ngx.DEBUG, "res is nil:")
        end
    else
        ngx.log(ngx.DEBUG, "cookie is nil:")
    end
    cache:close()
    local sign = _M.sign5s(duration)

    --[[local sign = _M.sign('cookie')
    if ngx.var.http_cookie ~= nil then
        if string.find( ngx.var.http_cookie , sign) ~= nil then
            return
        end
    end]]--

    if _M.verify_javascript_html == nil then
        local path = CdnConfig.home_path() .."/configs/verify_js.html"
        local f = io.open( path, 'r' )
        if f ~= nil then
            _M.verify_javascript_html = f:read("*all")
            f:close()
        end
    end
    
    local cookie_prefix = CdnConfig.configs['cookie_prefix']

    ngx.log(ngx.DEBUG, "不符合条件:"..sign..','..cookie_prefix)

    local redirect_to = nil
    local html = _M.verify_javascript_html
    -- local sign = _M.sign('challenge')
    html = string.gsub( html,'INFOCOOKIE', sign )
    html = string.gsub( html,'COOKIEPREFIX',cookie_prefix )

    if ngx.var.args ~= nil then
		redirect_to =  ngx.var.scheme.."://"..ngx.var.http_host..ngx.var.uri.."?"..ngx.var.args , ngx.HTTP_MOVED_TEMPORARILY
	else
		redirect_to =  ngx.var.scheme.."://"..ngx.var.http_host..ngx.var.uri , ngx.HTTP_MOVED_TEMPORARILY
	end

    html = util.string_replace( html,'INFOURI',redirect_to, 1 )
    
    ngx.header.content_type = "text/html"
    ngx.header['cache-control'] = "no-cache, no-store, must-revalidate"
    ngx.header['pragma'] = "no-cache"
    ngx.header['expires'] = "0"
    ngx.header.charset = "utf-8"
    ngx.var.verified = "1"
    ngx.say( html )
    
    ngx.exit(200)
end

function _M.filter(verify_case, duration)
    if ngx.var.verified == "1" then -- ngx.var.hit_firewall == "challenge" or ngx.var.hit_firewall == "js" or ngx.var.hit_firewall == "captcha" then
        return 
    end
    if verify_case == "captcha" then
        _M.verify_captcha(duration)
    end
    if verify_case == "cookie" then
        _M.verify_javascript(duration)
    end
    
    if verify_case == "js" then
        _M.verify_javascript(duration)
    end
    --[[
    if CdnConfig.site_configs["browser_verify_enable"] ~= true then
        return
    end
    
    local matcher_list = CdnConfig.site_configs['matcher']
    for i,rule in ipairs( CdnConfig.site_configs["browser_verify_rule"] ) do
        local enable = rule['enable']
        local matcher = matcher_list[ rule['matcher'] ] 
        if enable == true and request_tester.test( matcher ) == true then
            local verify_cookie,verify_javascript = false,false
            
            for idx,verify_type in ipairs( rule['type']) do
                if verify_type == 'cookie' then
                    verify_cookie = true
                elseif verify_type == 'javascript' then
                    verify_javascript = true
                end
            end


            return
        end
    end
    ]]--
end

return _M
