local CdnConfig = require "CdnConfig"
local util = require "util"
local _M = {}

local function replaceQuote(s)
    return '"' .. s:gsub('"', '\\"'):gsub('\n', '\\n'):gsub('\r', '\\r') .. '"'
end
function _M.log()
    local currentTime = os.date("%Y-%m-%d %H:%M:%S", os.time())
    currentTime = "\"" .. currentTime .. "\""

    local req_body = '-'
    if ngx.var.request_body then
        req_body = ngx.var.request_body
        -- req_body = string.gsub(req_body,"\n','")
        -- req_body = string.gsub(req_body,"\t','")
        req_body = util.truncateStringUTF8_FFI(req_body, 1000)
    end
    req_body =  util.utf8sub(req_body, 1 ,1000) -- "\"" .. req_body .. "\""
    -- ngx.log(ngx.DEBUG, util.truncateStringUTF8_FFI("你好，测试一下是否正常呢？？？？", 5))
    local req_status = 0
    if ngx.var.status then
        req_status = ngx.var.status
    end

    local req_time = 0
    if ngx.var.request_time then
        req_time = ngx.var.request_time
    end
    local is_https = ngx.var.is_https
    if is_https == 'yes' then
        is_https = 1
    else
        is_https = 0
    end
    local event = replaceQuote(ngx.var.eventsid)
    local req_domain = replaceQuote(ngx.var.host)
    local req_req = replaceQuote(ngx.unescape_uri(ngx.var.request_uri))
    local req_ua = replaceQuote(not ngx.var.http_user_agent and "" or ngx.var.http_user_agent)
    local req_method = replaceQuote(not ngx.var.request_method and "" or ngx.var.request_method)
    local remote_addr = replaceQuote(util.get_client_ip())
    local server_addr =  replaceQuote(not ngx.var.upstream_addr and "" or ngx.var.upstream_addr)
    local domain = replaceQuote(ngx.var.domain)
    -- 请求时间
    local request_time = tonumber(not ngx.var.request_time and 0 or ngx.var.request_time)
    
    local upstream_status = replaceQuote(not ngx.var.upstream_status and "" or ngx.var.upstream_status)

    local upstream_response_time = tonumber(not ngx.var.upstream_response_time and 0 or ngx.var.upstream_response_time)
    local upstream_response_length = tonumber(not ngx.var.upstream_response_length and 0 or ngx.var.upstream_response_length)

    local upstream_cache_status = replaceQuote(not ngx.var.upstream_cache_status and "" or ngx.var.upstream_cache_status)

    local hit_rule = replaceQuote(not ngx.var.hit_rule and "" or ngx.var.hit_rule)
    local hit_firewall = replaceQuote(not ngx.var.hit_firewall and "" or ngx.var.hit_firewall)
    local reject = replaceQuote(not ngx.var.reject and "no" or ngx.var.reject)
    local reject_reason = replaceQuote(not ngx.var.reject_reason and "" or ngx.var.reject_reason)
    local reject_log = replaceQuote(not ngx.var.reject_log and "" or ngx.var.reject_log)
    local no_cache = replaceQuote(not ngx.var.no_cache and "no" or (tostring(ngx.var.no_cache) == "0" and "no" or ngx.var.no_cache))
    
    local upstream_bytes_received = tonumber(not ngx.var.upstream_bytes_received and 0 or ngx.var.upstream_bytes_received)
    local upstream_bytes_send = tonumber(not ngx.var.upstream_bytes_send and 0 or ngx.var.upstream_bytes_send)
    local upstream_connect_time = tonumber(not ngx.var.upstream_connect_time and 0 or ngx.var.upstream_connect_time)
    local upstream_header_time = tonumber(not ngx.var.upstream_header_time and 0 or ngx.var.upstream_header_time)

    -- local myparams = ("("..event..','.. req_domain .. ',' ..remote_addr..','.. req_ua..','.. req_method..','.. server_addr..','..currentTime..','..request_time .. ','.. upstream_status..','..upstream_response_time..','.. upstream_response_length..','..ngx.var.body_bytes_sent..','..ngx.var.status..','..req_req..','..req_body..','..is_https..','..hit_rule..','..hit_firewall..','..reject..','..reject_reason..','..no_cache..','..domain..")")
    local myparams = ('{"eventId":'..event..
    ', "domain": '.. req_domain .. 
    ', "node": '..CdnConfig.configs['id'] ..
    ', "remote_addr": ' ..remote_addr..
    ', "ua": '.. req_ua..
    ', "method": '.. req_method..
    ', "server_addr": '.. server_addr..
    ', "timestamp": '..currentTime..
    ', "request_time": '..request_time .. 
    ', "upstream_status": '.. upstream_status..
    
    ', "upstream_bytes_received": '..upstream_bytes_received..
    ', "upstream_bytes_send": '..upstream_bytes_send..
    ', "upstream_connect_time": '..upstream_connect_time..
    ', "upstream_header_time": '..upstream_header_time..

    ', "upstream_response_time": '..upstream_response_time..
    ', "upstream_response_length": '.. upstream_response_length..
    ', "upstream_cache_status": '..upstream_cache_status..
    ', "body_bytes_sent": '..ngx.var.body_bytes_sent..
    ', "status": '..ngx.var.status..
    ', "req_uri": '..req_req..
    ', "req_body": '..req_body..
    ', "is_https": '..is_https..
    ', "hit_rule": '..hit_rule..
    ', "hit_firewall": '..hit_firewall..
    ', "reject": '..reject..
    ', "reject_reason": '..reject_reason..
    ', "reject_log": '..reject_log..
    ', "no_cache":'..no_cache..
    ',"domain_id":'..domain..'}')
    local key = "logs"
    ngx.log(ngx.DEBUG, myparams)
    -- local len,err = ngx.shared.logs:rpush(key, myparams)

    --[[if err then
        ngx.log(ngx.ERR,"failed to put log vals into shared dict")
        return
    end]]--
    local logger = require "resty.logger.socket"
    if not logger.initted() then
        local ok, err = logger.init{
            sock_type = CdnConfig.configs["fluent_protocol"],
            host = CdnConfig.configs['fluent'],
            port = tonumber(CdnConfig.configs['fluent_port']),
            flush_limit = 1024,
            drop_limit = 65535,
        }
        if not ok then
            ngx.log(ngx.ERR, "failed to initialize the logger: ",
                    err)
            return
        end
    else
        ngx.log(ngx.ERR, "log inited")
    end

    -- construct the custom access log message in
    -- the Lua variable "msg"
    -- local nsg = "{}"
    local bytes, err = logger.log(myparams)
    if err then
        ngx.log(ngx.ERR, "failed to log message: ", err)
        return
    end

end

CdnConfig.site_configs = {}

return _M